<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');
/**
 * ShortCodes
 *
 * @Version 0.75
 * @author Eugeny Panikarowsky - evgenii_panikaro@mail.ru
 * @copyright © 2015 Eugeny Panikarowsky
 *
*/
class ShortCode_Model extends Core_Entity
{
	
	/**
	 * Backend property
	 * @var mixed
	 */
	public $img = 1;
	
	/**
	 * Model name
	 * @var mixed
	 */
	protected $_modelName = 'shortcode';
	
	protected $_className = 'ShortCode_Model';

	static protected $_xml			= false;

	static protected $shortcode_tags = array();
	
	static protected $_old = NULL;
	
	static protected $_new = NULL;
	
	/**
	 * Constructor.
	 * @param int $id entity ID
	 */
	public function __construct($id = NULL)
	{
		parent::__construct($id);
	}
	
	static public function showmessage($message){
		if (Core_Auth::logged()) {
			return $message;
		} else {
			return '';
		}		
	}
	
	static public function checkXSL($attr) {

		if (!isset($attr['xsl']) or trim($attr['xsl'] == '')) {
			return self::showmessage(Core::_('shortcode.notxsl'));
		}

		$oXsl = Core_Entity::factory('Xsl')->getByName($attr['xsl']);
		if (is_null($oXsl)) {
			return self::showmessage(Core::_('shortcode.nonexsl'));
		}
		return $oXsl;

	}
	
	/**
	 * Change item status
	 * @return self
	 */
	public function changeActive()
	{
		$this->active = 1 - $this->active;
		$this->save();

		return $this;
	}
	
	static protected function loadShordcodes() {
		
		$aCustomShortCodes = (isset($_SESSION['shortcodes']) && is_array($_SESSION['shortcodes']) && !empty($_SESSION['shortcodes'])) ? $_SESSION['shortcodes'] : null;

		$oShortCodes = Core_Entity::factory('shortcode');
		$oShortCodes->queryBuilder()
			->where('site_id', '=', CURRENT_SITE)
			->where('active', '=', 1);
		$aShortCodes = $oShortCodes->findAll();

		if (!empty($aShortCodes)) {
			foreach ($aShortCodes as $oShortCode) {
				if (self::shortcode_exists($oShortCode->tag)) {
					continue;
				}
				$path = $oShortCode->getShortCodeFilePath();
				if (is_file($path)) {
					include_once ($path);
					$func = 'ShortCode_Handler_'.$oShortCode->tag;
					if ( is_callable($func) ) {
						self::add_shortcode($oShortCode->tag, $func);
					} else {
						echo self::showmessage(Core::_('shortcode.noexisthandlerfunc',$func));
					}
				} else {
					echo self::showmessage(Core::_('shortcode.noexisthandlerfile',$path));
				}
			}
		}
		
		if ($aCustomShortCodes) {
			foreach ($aCustomShortCodes as $tag => $func) {
				if ( is_callable($func) ) {
					self::add_shortcode($tag, $func);
				} else {
					echo self::showmessage(Core::_('shortcode.noexisthandlerfunc',$func));
				}
			}
		}
	}
	
	static public function onBeforeshowBody($object, $args) {
		$sContollerName = self::getControllerName();
		if ($sContollerName == 'default' || $sContollerName == 'Core_Command_Controller_Default') {
			ob_start();
		}
	}
	
	static public function onAfterProcess($object, $args) {
		
		Core_Event::notify(__CLASS__ . '.onBeforeDoShortCode', __CLASS__);
		
		$sContent = ob_get_clean();
		
		self::loadShordcodes();
		
		echo self::do_shortcode($sContent,true);
	}
	
	
	static private function getControllerName() {
		Shortcode_Router::add('robots.txt', '/robots.txt')
			->controller('Core_Command_Controller_Robots');

		Shortcode_Router::add('favicon.ico', '/favicon.ico')
			->controller('Core_Command_Controller_Favicon');

		Shortcode_Router::add('favicon.png', '/favicon.png')
			->controller('Core_Command_Controller_Favicon');

		Shortcode_Router::add('edit-in-place.php', '/edit-in-place.php')
			->controller('Core_Command_Controller_Edit_In_Place');

		Shortcode_Router::add('default', '()');
		
		//$sControllerName = '';

		$sControllerName = Shortcode_Router::factory(Core::$url['path'])->execute();
		
	//	var_dump($sControllerName);

		return $sControllerName;
	}
	
	
	static public function onAftershowBody($object, $args) {
		
		$sContollerName = self::getControllerName();
//		echo $sContollerName;
		if ($sContollerName != 'default' && $sContollerName != 'Core_Command_Controller_Default') {
			return;
		}
		
		
		
		Core_Event::notify(__CLASS__ . '.onBeforeDoShortCode', __CLASS__);
		
		if (HOSTCMS_UPDATE_NUMBER >= 132) {
			$sContent = $object->getBody();
		} else {
			$sContent = ob_get_clean();
		}
		
		self::loadShordcodes();

		$result = self::do_shortcode($sContent);
		
		$Core_Response = new Core_Response();
		$Core_Response
			->body($result)
			->header('X-Powered-By', Core::xPoweredBy())
			->sendHeaders()
			->showBody();
		
		die();
		if (get_class($object) == 'Core_Response' && $object->getStatus() != '301') {
		//	die();
		}
	}

	static public function add_shortcode($tag, $func) {

		if ( is_callable($func) ) {
			self::$shortcode_tags[$tag] = $func;
		} else {
			self::$shortcode_tags[$tag] = 'ShortCode_Model::blank';
		}
	}
	
	static protected function blank() {
		return '';
	}

	/**
	 * Removes hook for shortcode.
	 *
	 * @uses $shortcode_tags
	 *
	 * @param string $tag shortcode tag to remove hook for.
	 */
	static function remove_shortcode($tag) {
		unset(self::$shortcode_tags[$tag]);
	}

	/**
	 * Clear all shortcodes.
	 *
	 * This function is simple, it clears all of the shortcode tags by replacing the
	 * shortcodes global by a empty array. This is actually a very efficient method
	 * for removing all shortcodes.
	 *
	 * @uses $shortcode_tags
	 */
	static function remove_all_shortcodes() {
		self::$shortcode_tags = array();
	}

	/**
	 * Whether a registered shortcode exists named $tag
	 *
	 * @global array $shortcode_tags
	 * @param string $tag
	 * @return boolean
	 */
	static function shortcode_exists( $tag ) {
		return array_key_exists( $tag, self::$shortcode_tags );
	}

	/**
	 * Whether the passed content contains the specified shortcode
	 *
	 *
	 * @param string $tag
	 * @return boolean
	 */
	static function has_shortcode( $content, $tag ) {
		if ( false === strpos( $content, '[' ) ) {
			return false;
		}

		if ( shortcode_exists( $tag ) ) {
			preg_match_all( '/' . get_shortcode_regex() . '/s', $content, $matches, PREG_SET_ORDER );
			if ( empty( $matches ) ) {
				return false;
			}

			foreach ( $matches as $shortcode ) {
				if ( $tag === $shortcode[2] ) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Search content for shortcodes and filter shortcodes through their hooks.
	 *
	 * If there are no shortcode tags defined, then the content will be returned
	 * without any filtering. This might cause issues when plugins are disabled but
	 * the shortcode will still show up in the post or content.
	 *
	 * @uses $shortcode_tags
	 * @uses get_shortcode_regex() Gets the search pattern for searching shortcodes.
	 *
	 * @param string $content Content to search for shortcodes
	 * @return string Content with shortcodes filtered out.
	 */
	static function do_shortcode($content,$xml = false) {
		if ( false === strpos( $content, '[' ) ) {
			return $content;
		}
		

		if (empty(self::$shortcode_tags) || !is_array(self::$shortcode_tags)) {
			return $content;
		}

		$pattern = self::get_shortcode_regex();
		if ($xml) {
			$content = preg_replace_callback( "/$pattern/s", 'ShortCode_Model::do_shortcode_tag_xml', $content );
		} else {
			$content = preg_replace_callback( "/$pattern/s", 'ShortCode_Model::do_shortcode_tag', $content );
		}

		if ($xml) {
		//	return $content;
		}
		return html_entity_decode($content, 0 , 'UTF-8');
	}

	/**
	 * Retrieve the shortcode regular expression for searching.
	 *
	 * The regular expression combines the shortcode tags in the regular expression
	 * in a regex class.
	 *
	 * The regular expression contains 6 different sub matches to help with parsing.
	 *
	 * 1 - An extra [ to allow for escaping shortcodes with double [[]]
	 * 2 - The shortcode name
	 * 3 - The shortcode argument list
	 * 4 - The self closing /
	 * 5 - The content of a shortcode when it wraps some content.
	 * 6 - An extra ] to allow for escaping shortcodes with double [[]]
	 *
	 * @uses $shortcode_tags
	 *
	 * @return string The shortcode search regular expression
	 */
	static protected function get_shortcode_regex() {

		$tagnames = array_keys(self::$shortcode_tags);
		$tagregexp = join( '|', array_map('preg_quote', $tagnames) );

		// WARNING! Do not change this regex without changing do_shortcode_tag() and strip_shortcode_tag()
		// Also, see shortcode_unautop() and shortcode.js.
		return
			  '\\['                              // Opening bracket
			. '(\\[?)'                           // 1: Optional second opening bracket for escaping shortcodes: [[tag]]
			. "($tagregexp)"                     // 2: Shortcode name
			. '(?![\\w-])'                       // Not followed by word character or hyphen
			. '('                                // 3: Unroll the loop: Inside the opening shortcode tag
			.     '[^\\]\\/]*'                   // Not a closing bracket or forward slash
			.     '(?:'
			.         '\\/(?!\\])'               // A forward slash not followed by a closing bracket
			.         '[^\\]\\/]*'               // Not a closing bracket or forward slash
			.     ')*?'
			. ')'
			. '(?:'
			.     '(\\/)'                        // 4: Self closing tag ...
			.     '\\]'                          // ... and closing bracket
			. '|'
			.     '\\]'                          // Closing bracket
			.     '(?:'
			.         '('                        // 5: Unroll the loop: Optionally, anything between the opening and closing shortcode tags
			.             '[^\\[]*+'             // Not an opening bracket
			.             '(?:'
			.                 '\\[(?!\\/\\2\\])' // An opening bracket not followed by the closing shortcode tag
			.                 '[^\\[]*+'         // Not an opening bracket
			.             ')*+'
			.         ')'
			.         '\\[\\/\\2\\]'             // Closing shortcode tag
			.     ')?'
			. ')'
			. '(\\]?)';                          // 6: Optional second closing brocket for escaping shortcodes: [[tag]]
	}
	
	/**
	 * Regular Expression callable for do_shortcode() for calling shortcode hook.
	 * @see get_shortcode_regex for details of the match array contents.
	 *
	 * @since 2.5.0
	 * @access private
	 * @uses $shortcode_tags
	 *
	 * @param array $m Regular expression match array
	 * @return mixed False on failure.
	 */
	static function do_shortcode_tag_xml( $m ) {

		// allow [[foo]] syntax for escaping a tag
		if ( $m[1] == '[' && $m[6] == ']' ) {
			return substr($m[0], 1, -1);
		}

		$tag = $m[2];
		$attr = self::shortcode_parse_atts( $m[3] );

		$return = '';
		if ( isset( $m[5] ) ) {
			// enclosing tag - extra parameter
			$return  = $m[1] . call_user_func( self::$shortcode_tags[$tag], $attr, $m[5], $tag ) . $m[6];
		} else {
			// self-closing tag
			$return  = $m[1] . call_user_func( self::$shortcode_tags[$tag], $attr, null,  $tag ) . $m[6];
		}

		$return = str_replace("\r\n",'',$return);
		if (strstr($return,'hostcms:id')) {
			$return = preg_replace("/<p><div hostcms:id(.*?)>(.*?)<\/div><\/p>/s", '$2', $return);
		}
		$return = htmlspecialchars($return);
		
	//	$return = htmlentities($return, ENT_QUOTES,"UTF-8");
		
		return $return;
	}

	/**
	 * Regular Expression callable for do_shortcode() for calling shortcode hook.
	 * @see get_shortcode_regex for details of the match array contents.
	 *
	 * @since 2.5.0
	 * @access private
	 * @uses $shortcode_tags
	 *
	 * @param array $m Regular expression match array
	 * @return mixed False on failure.
	 */
	static function do_shortcode_tag( $m ) {

		// allow [[foo]] syntax for escaping a tag
		if ( $m[1] == '[' && $m[6] == ']' ) {
			return substr($m[0], 1, -1);
		}

		$tag = $m[2];
		$attr = self::shortcode_parse_atts( $m[3] );

		$return = '';
		if ( isset( $m[5] ) ) {
			// enclosing tag - extra parameter
			$return  = $m[1] . call_user_func( self::$shortcode_tags[$tag], $attr, $m[5], $tag ) . $m[6];
		} else {
			// self-closing tag
			$return  = $m[1] . call_user_func( self::$shortcode_tags[$tag], $attr, null,  $tag ) . $m[6];
		}
		return $return;
	}

	/**
	 * Retrieve all attributes from the shortcodes tag.
	 *
	 * The attributes list has the attribute name as the key and the value of the

	 * attribute as the value in the key/value pair. This allows for easier
	 * retrieval of the attributes, since all attributes have to be known.
	 *
	 * @param string $text
	 * @return array List of attributes and their value.
	 */
	static function shortcode_parse_atts($text) {
		$atts = array();
		$pattern = '/(\w+)\s*=\s*"([^"]*)"(?:\s|$)|(\w+)\s*=\s*\'([^\']*)\'(?:\s|$)|(\w+)\s*=\s*([^\s\'"]+)(?:\s|$)|"([^"]*)"(?:\s|$)|(\S+)(?:\s|$)/';
		$text = preg_replace("/[\x{00a0}\x{200b}]+/u", " ", $text);
		if ( preg_match_all($pattern, $text, $match, PREG_SET_ORDER) ) {
			foreach ($match as $m) {
				if (!empty($m[1])) {
					$atts[strtolower($m[1])] = stripcslashes($m[2]);
				} elseif (!empty($m[3])) {
					$atts[strtolower($m[3])] = stripcslashes($m[4]);
				} elseif (!empty($m[5])) {
					$atts[strtolower($m[5])] = stripcslashes($m[6]);
				} elseif (isset($m[7]) and strlen($m[7])) {
					$atts[] = stripcslashes($m[7]);
				} elseif (isset($m[8])) {
					$atts[] = stripcslashes($m[8]);
				}
			}
		} else {
			$atts = ltrim($text);
		}
		return $atts;
	}
	
	public function getShortCodesPath() {
		return CMS_FOLDER . 'hostcmsfiles'. DIRECTORY_SEPARATOR . 'shortcode'. DIRECTORY_SEPARATOR;
	}
	
	/**
	 * Get configuration file path
	 * @return string
	 */
	public function getShortCodeFilePath()
	{
		return $this->getShortCodesPath() . "short_"  . intval($this->id) . ".php";
	}
	
	/**
	 * Delete object from database
	 * @param mixed $primaryKey primary key for deleting object
	 * @return Core_Entity
	 */
	public function delete($primaryKey = NULL)
	{
		if (is_null($primaryKey))
		{
			$primaryKey = $this->getPrimaryKey();
		}

		$this->id = $primaryKey;

		try
		{
			Core_File::delete($this->getShortCodeFilePath());
		} catch (Exception $e) {}

		return parent::delete($primaryKey);
	}
	
	/**
	 * Get config file content
	 * @return string|NULL
	 */
	public function loadShortCodeFile()
	{
		$path = $this->getShortCodeFilePath();

		if (is_file($path))
		{
			return Core_File::read($path);
		}
		else
		{
			return NULL;
		}
	}
	
	/**
	 * Save config content
	 * @param string $content content
	 */
	public function saveShortCodeFile($content)
	{
		$this->save();

		$content = trim($content);

		Core_File::mkdir( dirname($path = $this->getShortCodeFilePath()), CHMOD, TRUE);
		
		Core_File::write($path, $content);
	}

	/**
	 * Combine user attributes with known attributes and fill in defaults when needed.
	 *
	 * The pairs should be considered to be all of the attributes which are
	 * supported by the caller and given as a list. The returned attributes will
	 * only contain the attributes in the $pairs list.
	 *
	 * If the $atts list has unsupported attributes, then they will be ignored and
	 * removed from the final returned list.
	 *
	 * @param array $pairs Entire list of supported attributes and their defaults.
	 * @param array $atts User defined attributes in shortcode tag.
	 * @return array Combined attribute list.
	 */
	function shortcode_atts( $pairs, $atts ) {
		$atts = (array)$atts;
		$out = array();
		foreach($pairs as $name => $default) {
			if ( array_key_exists($name, $atts) ) {
				$out[$name] = $atts[$name];
			} else {
				$out[$name] = $default;
			}
		}

		return $out;
	}

	/**
	 * Remove all shortcode tags from the given content.
	 *
	 *
	 * @uses $shortcode_tags
	 *
	 * @param string $content Content to remove shortcode tags.
	 * @return string Content without shortcode tags.
	 */
	static public function strip_shortcodes( $content ) {

		if ( false === strpos( $content, '[' ) ) {
			return $content;
		}

		if (empty(self::$shortcode_tags) || !is_array(self::$shortcode_tags)) {
			return $content;
		}

		$pattern = self::get_shortcode_regex();

		return preg_replace_callback( "/$pattern/s", 'ShortCode_Model::strip_shortcode_tag', $content );
	}

	function strip_shortcode_tag( $m ) {
		// allow [[foo]] syntax for escaping a tag
		if ( $m[1] == '[' && $m[6] == ']' ) {
			return substr($m[0], 1, -1);
		}

		return $m[1] . $m[6];
	}


}